import { Box, CircularProgress, Typography, Paper, Chip, withStyles, colors } from '@material-ui/core';
import {ChipProps} from '@material-ui/core/Chip';
import React, { useEffect, useState } from 'react';
import {QueryListOption,Content} from '../../model/contents';
import {ClientContentAccessesor} from '../accessor/contents';
import {useTheme, makeStyles, Theme, emphasize, fade} from '@material-ui/core/styles';
import {blue , pink} from '@material-ui/core/colors';

type TagChipStyleProp = {
    color: string
}

const useTagStyles = makeStyles((theme:Theme)=>({
    root:(props:TagChipStyleProp)=>({
        color: theme.palette.getContrastText(props.color),
        backgroundColor: props.color,
    }),
    clickable:(props:TagChipStyleProp)=>({
        '&:hover, &:focus':{
            backgroundColor:emphasize(props.color,0.08)
        }
    }),
    deletable: {
        '&:focus': {
            backgroundColor: (props:TagChipStyleProp)=>emphasize(props.color, 0.2),
        }
    },
    outlined:{
        color: (props:TagChipStyleProp)=>props.color,
        border: (props:TagChipStyleProp)=> `1px solid ${props.color}`,
        '$clickable&:hover, $clickable&:focus, $deletable&:focus': {
            backgroundColor:(props:TagChipStyleProp)=>fade(props.color,theme.palette.action.hoverOpacity),
        },
    },
    icon:{
        color:"inherit",
    },
    deleteIcon:{
        color:(props:TagChipStyleProp)=>fade(theme.palette.getContrastText(props.color),0.7),
        "&:hover, &:active":{
            color:(props:TagChipStyleProp)=>theme.palette.getContrastText(props.color),
        }
    }
}));

const getTagColorName = (tagname :string):string=>{
    if(tagname.startsWith("female")){
        return pink['700'];
    }
    else if(tagname.startsWith("male")){
        return blue[600];
    }
    else return "default";
}

const TagChip = (props:Omit<ChipProps,"color"> & {color: string})=>{
    const {color,...rest} = props;
    const classes = useTagStyles({color : color !== "default" ? color : "#000"});
    return <Chip color="default" classes={{
        ...(color !== "default" ? classes : {})
    }} {...rest}></Chip>;
}

const useStyles = makeStyles((theme:Theme)=>({
    root:{
        display:"grid",
        gridGap: theme.spacing(4),
    },
    table_thumbnail:{
        width: 200,
        height: 200,
        background: '#272733',
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
    },
    contentPaper:{
        display:'flex',
        height:200,
    },
    content_thumnail: {
        maxWidth: '100%',
        maxHeight: '100%',
    },
    content_info:{
        padding: `${theme.spacing(1)}px ${theme.spacing(2)}px`,
        width: '100%',
        display: 'flex',
        flexDirection: 'column',
    },
    tag_list:{
        display: 'flex',
        justifyContent: 'flex-start',
        flexWrap: 'wrap',
        '& > *': {
            margin: theme.spacing(0.5),
        },
          
    }
}));

export type GalleryProp = {
    option?:QueryListOption;
};
type GalleryState = {
    content:Content[]|undefined;
}
const content_accessor = new ClientContentAccessesor;

export const Gallery = (props: GalleryProp)=>{
    const [state,setState]= useState<GalleryState>({content:undefined});
    useEffect(()=>{
        (async ()=>{
            const c = await content_accessor.findList(props.option);
            setState({content:c});
        })()
    },[props.option]);
    const num = 1;
    const classes = useStyles();
    const theme = useTheme();
        if(state.content === undefined){
            return (<div><Box>
                <CircularProgress title="loading"/>
                </Box></div>);
        }
        else{
            return (<div className={classes.root}>{
                state.content.map(x=>{
                    const thumbnail_url = `/content/${x.id}/${x.content_type}/thumbnail`;
                    return (<Paper key={x.id} elevation={4} className={classes.contentPaper}>
                        <a className={classes.table_thumbnail}>{
                            x.content_type === "manga" ? <img className={classes.content_thumnail} src={thumbnail_url}></img> :
                            <video className={classes.content_thumnail} src={thumbnail_url}></video>
                        }</a>
                        <Box className={classes.content_info}>
                            <Typography variant="h5" style={{marginLeft:theme.spacing(2)}}>{x.title}</Typography>
                            <Box className={classes.tag_list}>
                                {x.tags.map(x=>{
                                    const tagcolor = getTagColorName(x);
                                    console.log(tagcolor);
                                    return (<TagChip key={x} label={x} clickable={true} color={tagcolor}></TagChip>);
                            })}
                            </Box>
                        </Box>
                    </Paper>);
                })
                }
                </div>);
        }
}